% test_lyap
%
% Computation of the Cholesky factor R of the solution Q=R*R' of the 
% Lyapunov equation
clear all;
figure(1); clf; 
tol = 1e-12;

% System matrices
load('build.mat')
A=full(A);B=full(B);C=full(C);D=0;
n=length(A);
ev = eig(A);
subplot(2,2,1)
plot(real(ev), imag(ev), 'b*');
title('eigenvalues of A')

% frequency response G(iw)
disp('Computation of the frequency response');
wmin  = 0;
wmax  = 2;
numw  = 100;
wh    = (wmax-wmin)/numw;
w     = logspace(wmin,wmax,numw);
i     = sqrt(-1);
for j = 1:numw
   iwj = i*w(j);
   G   = sparse(C)*(sparse(iwj*speye(n)-A)\sparse(B));
   nrmG(j) = norm(full(G));
end 
subplot(2,2,2);
semilogx(w, nrmG(:), 'LineWidth', 1.5);
xlabel('Frequency w');
ylabel('Magnitude');
title('Frequency response G(iw)');

% Solution of the Lyapunov equation A`*Q+Q*A=-C`*C
disp('  ');
disp('Solution of the Lyapunov equation A`*Q+Q*A=-C`*C');
[R, convC] = sfm_lyap(A, C, tol);
nresC      = norm(A'*R*R'+R*R'*A+C'*C,'fro')/norm(C,'fro')^2
subplot(2,2,3);
semilogy(1:length(convC), convC, 'b*-');
xlim([1 length(convC)]);
xlabel('Iteration');
ylabel('||Ai+I||_F');
title('Convergence of the Sign function method');

% Solution of the Lyapunov equation A*P+P*A`=-B*B`
disp('  ');
disp('Solution of the Lyapunov equation A*P+P*A`=-B*B`');
% ATTENTION!!! 
% Uncomment this row and insert the right matrices
% [S, convB] = sfm_lyap(??, ??, tol);
nresB = norm(A*S*S'+S*S'*A'+B*B','fro')/norm(B,'fro')^2
subplot(2,2,4);
semilogy(1:length(convB), convB, 'b*-');
xlim([1 length(convB)]);
xlabel('Iteration');
ylabel('||Ai+I||_F');
title('Convergence of the sign function method');